This is for board Supermicro H8scm. The code was done by existing chips and
superiotool.

WPCM450 is more like an EC. SuperIO is just a part of multi-features.

Signed-off-by: Zheng Bao <zheng.bao@amd.com>

Index: src/superio/Kconfig
===================================================================
--- src/superio/Kconfig	(revision 6459)
+++ src/superio/Kconfig	(working copy)
@@ -21,6 +21,7 @@
 source src/superio/intel/Kconfig
 source src/superio/ite/Kconfig
 source src/superio/nsc/Kconfig
+source src/superio/nuvoton/Kconfig
 source src/superio/renesas/Kconfig
 source src/superio/serverengines/Kconfig
 source src/superio/smsc/Kconfig
Index: src/superio/Makefile.inc
===================================================================
--- src/superio/Makefile.inc	(revision 6459)
+++ src/superio/Makefile.inc	(working copy)
@@ -21,6 +21,7 @@
 subdirs-y += intel
 subdirs-y += ite
 subdirs-y += nsc
+subdirs-y += nuvoton
 subdirs-y += renesas
 #subdirs-y += serverengines
 subdirs-y += smsc
Index: src/superio/nuvoton/wpcm450/early_init.c
===================================================================
--- src/superio/nuvoton/wpcm450/early_init.c	(revision 0)
+++ src/superio/nuvoton/wpcm450/early_init.c	(revision 0)
@@ -0,0 +1,51 @@
+/*
+ * This file is part of the coreboot project.
+ *
+ * Copyright (C) 2011 Advanced Micro Devices, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ */
+
+#include <arch/romcc_io.h>
+#include "wpcm450.h"
+
+static void wpcm450_pnp_set_logical_device(u8 dev, u16 port)
+{
+	outb(0x7, port);
+	outb(dev, port+1);
+}
+
+static void wpcm450_pnp_set_enable(u8 dev, u16 port, u8 enable)
+{
+	outb(0x30, port);
+	outb(enable, port+1);
+}
+
+static void wpcm450_pnp_set_iobase(u8 dev, u16 port, u8 index, u16 iobase)
+{
+	outb(index, port);
+	outb((iobase>>8)&0xFF, port+1);
+	outb(index+1, port);
+	outb(iobase&0xFF, port+1);
+}
+
+static void wpcm450_enable_dev(u8 dev, u16 port, u16 iobase)
+{
+	wpcm450_pnp_set_logical_device(dev, port);
+	wpcm450_pnp_set_enable(dev, port, 0);
+	if (iobase)
+		wpcm450_pnp_set_iobase(dev, port, PNP_IDX_IO0, iobase);
+	wpcm450_pnp_set_enable(dev, port, 1);
+}
Index: src/superio/nuvoton/wpcm450/Makefile.inc
===================================================================
--- src/superio/nuvoton/wpcm450/Makefile.inc	(revision 0)
+++ src/superio/nuvoton/wpcm450/Makefile.inc	(revision 0)
@@ -0,0 +1,22 @@
+##
+## This file is part of the coreboot project.
+##
+## Copyright (C) 2011 Advanced Micro Devices, Inc.
+##
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+##
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with this program; if not, write to the Free Software
+## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+##
+
+ramstage-$(CONFIG_SUPERIO_NUVOTON_WPCM450) += superio.c
+
Index: src/superio/nuvoton/wpcm450/wpcm450.h
===================================================================
--- src/superio/nuvoton/wpcm450/wpcm450.h	(revision 0)
+++ src/superio/nuvoton/wpcm450/wpcm450.h	(revision 0)
@@ -0,0 +1,28 @@
+/*
+ * This file is part of the coreboot project.
+ *
+ * Copyright (C) 2011 Advanced Micro Devices, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ */
+
+#ifndef SUPERIO_NUVOTON_WPCM450_WPCM450_H
+#define SUPERIO_NUVOTON_WPCM450_WPCM450_H
+
+#define WPCM450_SP2  0x02 /* Com2 */
+#define WPCM450_SP1  0x03 /* Com1 */
+#define WPCM450_KBCK 0x06 /* Keyboard */
+
+#endif
Index: src/superio/nuvoton/wpcm450/superio.c
===================================================================
--- src/superio/nuvoton/wpcm450/superio.c	(revision 0)
+++ src/superio/nuvoton/wpcm450/superio.c	(revision 0)
@@ -0,0 +1,78 @@
+/*
+ * This file is part of the coreboot project.
+ *
+ * Copyright (C) 2011 Advanced Micro Devices, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ */
+
+#include <arch/io.h>
+#include <device/device.h>
+#include <device/pnp.h>
+#include <console/console.h>
+#include <string.h>
+#include <bitops.h>
+#include <uart8250.h>
+#include <pc80/keyboard.h>
+#include <stdlib.h>
+#include "chip.h"
+#include "wpcm450.h"
+
+static void init(device_t dev)
+{
+	struct superio_nuvoton_wpcm450_config *conf = dev->chip_info;
+	struct resource *res0;
+
+	if (!dev->enabled)
+		return;
+
+	switch(dev->path.pnp.device) {
+	case WPCM450_SP1:
+		res0 = find_resource(dev, PNP_IDX_IO0);
+		init_uart8250(res0->base, &conf->com1);
+		break;
+	case WPCM450_SP2:
+		res0 = find_resource(dev, PNP_IDX_IO0);
+		init_uart8250(res0->base, &conf->com2);
+		break;
+	case WPCM450_KBCK:
+		pc_keyboard_init(&conf->keyboard);
+		break;
+	}
+}
+
+static struct device_operations ops = {
+	.read_resources   = pnp_read_resources,
+	.set_resources    = pnp_set_resources,
+	.enable_resources = pnp_enable_resources,
+	.enable           = pnp_enable,
+	.init             = init,
+};
+
+static struct pnp_info pnp_dev_info[] = {
+	{ &ops, WPCM450_SP2,  PNP_IO0 | PNP_IRQ0 | PNP_DRQ0 | PNP_DRQ1, {0x07f8, 0}, },
+	{ &ops, WPCM450_SP1,  PNP_IO0 | PNP_IRQ0, {0x07f8, 0}, },
+	{ &ops, WPCM450_KBCK, PNP_IO0 | PNP_IO1 | PNP_IRQ0, {0x07f8, 0}, {0x07f8, 4}, },
+};
+
+static void enable_dev(struct device *dev)
+{
+	pnp_enable_devices(dev, &ops, ARRAY_SIZE(pnp_dev_info), pnp_dev_info);
+}
+
+struct chip_operations superio_nuvoton_wpcm450_ops = {
+	CHIP_NAME("NUVOTON WPCM450 Super I/O")
+	.enable_dev = enable_dev,
+};
Index: src/superio/nuvoton/wpcm450/chip.h
===================================================================
--- src/superio/nuvoton/wpcm450/chip.h	(revision 0)
+++ src/superio/nuvoton/wpcm450/chip.h	(revision 0)
@@ -0,0 +1,35 @@
+/*
+ * This file is part of the coreboot project.
+ *
+ * Copyright (C) 2011 Advanced Micro Devices, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ */
+
+#ifndef SUPERIO_NUVOTON_WPCM450_CHIP_H
+#define SUPERIO_NUVOTON_WPCM450_CHIP_H
+
+struct chip_operations;
+extern struct chip_operations superio_nuvoton_wpcm450_ops;
+
+#include <pc80/keyboard.h>
+#include <uart8250.h>
+
+struct superio_nuvoton_wpcm450_config {
+	struct uart8250 com1, com2;
+	struct pc_keyboard keyboard;
+};
+
+#endif
Index: src/superio/nuvoton/Kconfig
===================================================================
--- src/superio/nuvoton/Kconfig	(revision 0)
+++ src/superio/nuvoton/Kconfig	(revision 0)
@@ -0,0 +1,21 @@
+##
+## This file is part of the coreboot project.
+##
+## Copyright (C) 2011 Advanced Micro Devices, Inc.
+##
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; version 2 of the License.
+##
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with this program; if not, write to the Free Software
+## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+##
+
+config SUPERIO_NUVOTON_WPCM450
+	bool
Index: src/superio/nuvoton/Makefile.inc
===================================================================
--- src/superio/nuvoton/Makefile.inc	(revision 0)
+++ src/superio/nuvoton/Makefile.inc	(revision 0)
@@ -0,0 +1,20 @@
+##
+## This file is part of the coreboot project.
+##
+## Copyright (C) 2011 Advanced Micro Devices, Inc.
+##
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; version 2 of the License.
+##
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with this program; if not, write to the Free Software
+## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+##
+
+subdirs-y += wpcm450
